@@ -30,8 +30,8 @@ from congress import version
 LOG = logging.getLogger(__name__)
 
 core_opts = [
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help="The host IP to bind to"),
+    cfg.HostAddressOpt('bind_host', default='0.0.0.0',
+                       help="The host IP to bind to"),
     cfg.PortOpt('bind_port', default=1789,
                 help="The port to bind to"),
     cfg.IntOpt('max_simultaneous_requests', default=1024,
@@ -75,6 +75,10 @@ core_opts = [
     cfg.BoolOpt('replicated_policy_engine', default=False,
                 help='Set the flag to use congress with replicated policy '
                      'engines.'),
+    cfg.StrOpt('policy_library_path', default='/etc/congress/library',
+               help=_('The directory containing library policy files.')),
+    cfg.StrOpt('encryption_key_path', default='/etc/congress/keys',
+               help=_('The directory containing encryption keys.')),
     cfg.BoolOpt('distributed_architecture',
                 deprecated_for_removal=True,
                 deprecated_reason='distributed architecture is now the only '
@@ -154,7 +158,7 @@ def find_paste_config():
         raise cfg.ConfigFilesNotFoundError(
             config_files=[cfg.CONF.api_paste_config])
     config_path = os.path.abspath(config_path)
-    LOG.info(_("Config paste file: %s"), config_path)
+    LOG.info(("Config paste file: %s"), config_path)
     return config_path
 
 
