@@ -17,37 +17,31 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
-# Use new deepsix when appropriate
-from oslo_config import cfg
-if (hasattr(cfg.CONF, 'distributed_architecture')
-   and cfg.CONF.distributed_architecture):
-    from congress.dse2 import deepsix2 as deepsix
-else:
-    from congress.dse import deepsix
-
+import datetime
 from functools import cmp_to_key
 from functools import reduce
+import hashlib
+import json
+import time
 
 import eventlet
 from oslo_log import log as logging
+from oslo_utils import strutils
 import six
 
 from congress.datalog import compile
 from congress.datalog import utility
 from congress.datasources import datasource_utils as ds_utils
+from congress.db import db_ds_table_data
 from congress.dse2 import data_service
 from congress import exception
 from congress import utils
 
-import datetime
-import hashlib
-import json
-
 
 LOG = logging.getLogger(__name__)
 
 
-class DataSourceDriver(deepsix.deepSix):
+class DataSourceDriver(data_service.DataService):
     """A super-class for datasource drivers.
 
     This class implements a polling mechanism for polling a datasource.
@@ -130,7 +124,7 @@ class DataSourceDriver(deepsix.deepSix):
       that field-translator will cause the creation of a second table.  The
       field-translator will populate the second table, and each row in the
       primary table will (in the column for that field) contain a hash of the
-      second table's entries derived from the the primary table's row.  For
+      second table's entries derived from the primary table's row.  For
       example, if the translator is:
 
       {'translation-type': 'HDICT',
@@ -289,12 +283,17 @@ class DataSourceDriver(deepsix.deepSix):
 
     TRANSLATORS = []
 
-    def __init__(self, name, keys, inbox, datapath, args):
+    def __init__(self, name='', args=None):
+        self.name = name
+        self.type = 'datasource_driver'
         self.initialized = False
         self.last_updated_time = None
         self.last_error = None
         self.number_of_updates = 0
-
+        # TODO(haht): require ds_id/uuid argument here and maybe in DataService
+        #             best done along with cleaning out the dse1-related params
+        #             now made unnecessary
+        self.ds_id = args.get('ds_id') if args is not None else None
         # a dictionary from tablename to the SET of tuples, both currently
         #  and in the past.
         self.prior_state = dict()
@@ -322,20 +321,19 @@ class DataSourceDriver(deepsix.deepSix):
         self._table_deps = {}
 
         # setup translators here for datasource drivers that set TRANSLATORS.
-        for translator in self.TRANSLATORS:
-            self.register_translator(translator)
+        self.initialize_translators()
 
         # Make sure all data structures above are set up *before* calling
         #   this because it will publish info to the bus.
-        super(DataSourceDriver, self).__init__(name, keys, inbox, datapath)
+        super(DataSourceDriver, self).__init__(name)
 
         # For DSE2.  Must go after __init__
         if hasattr(self, 'add_rpc_endpoint'):
             self.add_rpc_endpoint(DataSourceDriverEndpoints(self))
 
     def get_snapshot(self, table_name):
-        print("datasource_driver get_snapshot(%s); %s" % (
-            table_name, self.state))
+        LOG.debug("datasource_driver get_snapshot(%s); %s",
+                  table_name, self.state)
         return self.state.get(table_name, set())
 
     def _make_tmp_state(self, root_table_name, row_data):
@@ -442,11 +440,15 @@ class DataSourceDriver(deepsix.deepSix):
 
         # check that translation_type is valid
         if translation_type not in self.VALID_TRANSLATION_TYPES:
-            msg = ("Translation Type %s not a valid transltion-type %s" % (
+            msg = ("Translation Type %s not a valid translation-type %s" % (
                    translation_type, self.VALID_TRANSLATION_TYPES))
             raise exception.InvalidTranslationType(msg)
         self._validate_by_translation_type(translator, related_tables)
 
+    def initialize_translators(self):
+        for translator in self.TRANSLATORS:
+            self.register_translator(translator)
+
     def register_translator(self, translator):
         """Registers translator with congress and validates its schema."""
         related_tables = []
@@ -1121,10 +1123,6 @@ class DataSourceDriver(deepsix.deepSix):
     def request_refresh(self):
         raise NotImplementedError('request_refresh() is not implemented.')
 
-    def cleanup(self):
-        """Cleanup this object in preparation for elimination."""
-        pass
-
     def get_status(self):
         d = {}
         d['last_updated'] = str(self.last_updated_time)
@@ -1178,6 +1176,9 @@ class DataSourceDriverEndpoints(data_service.DataServiceEndPoints):
     def request_refresh(self, context, source_id):
         return self.service.request_refresh()
 
+    def request_execute(self, context, action, action_args, wait):
+        return self.service.request_execute(context, action, action_args, wait)
+
 
 class PushedDataSourceDriver(DataSourceDriver):
     """Push Type DataSource Driver.
@@ -1185,13 +1186,24 @@ class PushedDataSourceDriver(DataSourceDriver):
     This DataSource Driver is a base class for push type datasource driver.
     """
 
-    def __init__(self, name, keys, inbox, datapath, args):
-        super(PushedDataSourceDriver, self).__init__(name, keys, inbox,
-                                                     datapath, args)
+    def __init__(self, name='', args=None):
+        self.persist_data = False
+        super(PushedDataSourceDriver, self).__init__(name, args)
 
         # For DSE2.  Must go after __init__
         if hasattr(self, 'add_rpc_endpoint'):
             self.add_rpc_endpoint(PushedDataSourceDriverEndpoints(self))
+
+        if args is not None:
+            self.persist_data = strutils.bool_from_string(
+                args.get('persist_data', 'False'), strict=True)
+
+        if self.persist_data:
+            if self.ds_id is None:
+                raise Exception('Push-type datasource driver does not have ID')
+            else:
+                self._restore_persisted_data()
+
         self.initialized = True
 
     def request_refresh(self):
@@ -1200,30 +1212,51 @@ class PushedDataSourceDriver(DataSourceDriver):
         # overrided in a subclass.
         pass
 
+    # Note (thread-safety): blocking function
+    def _restore_persisted_data(self):
+        self.state = {}
+        # Note (thread-safety): blocking call
+        data = db_ds_table_data.get_ds_table_data(self.ds_id)
+        for table in data:
+            self.state[table['tablename']] = table['tabledata']
+
+    # Note (thread-safety): blocking function
     def update_entire_data(self, table_id, objs):
-        LOG.info('update %s table in %s datasource' % (table_id, self.name))
+        LOG.info('update %s table in %s datasource', table_id, self.name)
         translator = self.get_translator(table_id)
         tablename = translator['table-name']
         self.prior_state = dict(self.state)
         self._update_state(
             tablename, PushedDataSourceDriver.convert_objs(objs, translator))
-        LOG.debug('publish a new state %s in %s' %
-                  (self.state[tablename], tablename))
+        LOG.debug('publish a new state %s in %s',
+                  self.state[tablename], tablename)
+        # Note (thread-safety): blocking call
         self.publish(tablename, self.state[tablename])
         self.number_of_updates += 1
         self.last_updated_time = datetime.datetime.now()
 
+        # persist in DB
+        if self.persist_data:
+            if self.ds_id is not None:
+                # Note (thread-safety): blocking call
+                db_ds_table_data.store_ds_table_data(
+                    self.ds_id, tablename, self.state[tablename])
+            else:
+                raise Exception('Push-type datasource driver does not have ID')
+
 
 class PushedDataSourceDriverEndpoints(data_service.DataServiceEndPoints):
     def __init__(self, service):
         super(PushedDataSourceDriverEndpoints, self).__init__(service)
 
+    # Note (thread-safety): blocking function
     def update_entire_data(self, context, table_id, source_id, objs):
+        # Note (thread-safety): blocking call
         return self.service.update_entire_data(table_id, objs)
 
 
 class PollingDataSourceDriver(DataSourceDriver):
-    def __init__(self, name, keys, inbox, datapath, args):
+    def __init__(self, name='', args=None):
         if args is None:
             args = dict()
 
@@ -1232,15 +1265,21 @@ class PollingDataSourceDriver(DataSourceDriver):
         else:
             poll_time = 10
 
-        # a number of tests rely on polling being disabled if there's no inbox
-        # provided to the deepSix base class so clamp to zero here in that case
-        self.poll_time = poll_time if inbox is not None else 0
+        self.poll_time = poll_time
+
+        self.lazy_tables = args.get('lazy_tables', [])
+        self.validate_lazy_tables()
+
+        # a dict for update method
+        # key: root table name
+        # value: update method
+        # ex: {'servers': <pointer for the updating method>}
+        self.update_methods = {}
 
         self.refresh_request_queue = eventlet.Queue(maxsize=1)
         self.worker_greenthread = None
 
-        super(PollingDataSourceDriver, self).__init__(name, keys, inbox,
-                                                      datapath, args)
+        super(PollingDataSourceDriver, self).__init__(name, args=args)
 
     def _init_end_start_poll(self):
         """Mark initializes the success and launch poll loop.
@@ -1248,30 +1287,101 @@ class PollingDataSourceDriver(DataSourceDriver):
         Every instance of this class must call the method at the end of
         __init__()
         """
-        LOG.debug("start to poll from datasource %s", self.name)
-        self.worker_greenthread = eventlet.spawn(self.poll_loop,
-                                                 self.poll_time)
+        if self._running:
+            self.worker_greenthread = eventlet.spawn(self.poll_loop,
+                                                     self.poll_time)
         self.initialized = True
 
-    def cleanup(self):
-        """Delete worker thread if created."""
-        if self.worker_greenthread is not None:
-            eventlet.greenthread.kill(self.worker_greenthread)
-            self.worker_greenthread = None
-            self.log_info("killed worker thread")
+    def add_update_method(self, method, translator):
+        if translator[self.TABLE_NAME] in self.update_methods:
+            raise exception.Conflict('A method has already registered for '
+                                     'the table %s.' %
+                                     translator[self.TABLE_NAME])
+        self.update_methods[translator[self.TABLE_NAME]] = method
 
-        super(PollingDataSourceDriver, self).cleanup()
+    def validate_lazy_tables(self):
+        """Check all the lazy_tables is root table name."""
+        root_table_names = [t[self.TABLE_NAME] for t in self.TRANSLATORS]
+        invalid_table = [t for t in self.lazy_tables
+                         if t not in root_table_names]
+        if invalid_table:
+            LOG.info('Invalid table name in lazy_tables config: %s')
+            msg = ("Invalid lazy tables: %s. Accepted tables for datasource "
+                   "%s are %s." % (invalid_table, self.name, root_table_names))
+            raise exception.BadRequest(msg)
+
+    def initialize_translators(self):
+        """Register translators for polling and define tables.
+
+        This registers a translator and defines tables for subscribers.
+        When a table name in root translator is specified as a lazy
+        it skips registering the translator and doesn't define the table.
+        """
+        for translator in self.TRANSLATORS:
+            if translator[self.TABLE_NAME] not in self.lazy_tables:
+                LOG.debug('register translator: %s'
+                          % translator[self.TABLE_NAME])
+                self.register_translator(translator)
+
+    def start(self):
+        super(PollingDataSourceDriver, self).start()
+        if not self.worker_greenthread:
+            self.worker_greenthread = eventlet.spawn(self.poll_loop,
+                                                     self.poll_time)
+
+    # Note(thread-safety): blocking function
+    def stop(self):
+        # Note(thread-safety): blocking call
+        self.stop_polling_thread()
+        super(PollingDataSourceDriver, self).stop()
+
+    # Note(thread-safety): blocking function
+    def stop_polling_thread(self):
+        if self.worker_greenthread is not None:
+            # Note(thread-safety): blocking call
+            eventlet.greenthread.kill(self.worker_greenthread)
+            try:
+                self.worker_greenthread.wait()
+            except eventlet.support.greenlets.GreenletExit:
+                pass
+            self.worker_greenthread = None
+            LOG.info("killed %s polling worker thread", self.name)
+
+    def get_snapshot(self, table_name):
+        """Return a snapshot of table."""
+        if (table_name in [t[self.TABLE_NAME] for t in self.TRANSLATORS] and
+                table_name not in self._table_deps):
+            new_translator = next(t for t in self.TRANSLATORS
+                                  if t[self.TABLE_NAME] == table_name)
+            self.register_translator(new_translator)
+            self.update_methods[table_name]()
+
+        return super(PollingDataSourceDriver, self).get_snapshot(table_name)
+
+    def get_row_data(self, table_id, *args, **kwargs):
+        if table_id not in self.state and table_id in self.lazy_tables:
+            raise exception.LazyTable(lazy_table=table_id)
+
+        return super(PollingDataSourceDriver, self).get_row_data(table_id,
+                                                                 *args,
+                                                                 **kwargs)
 
     def get_last_updated_time(self):
         return self.last_updated_time
 
+    def update_from_datasource(self):
+        for registered_table in self._table_deps:
+            LOG.debug('update table %s.' % registered_table)
+            self.update_methods[registered_table]()
+
+    # Note(thread-safety): blocking function
     def poll(self):
         """Periodically called to update new info.
 
         Function called periodically to grab new information, compute
         deltas, and publish those deltas.
         """
-        self.log_info("polling")
+        LOG.info("%s:: polling", self.name)
         self.prior_state = dict(self.state)  # copying self.state
         self.last_error = None  # non-None only when last poll errored
         try:
@@ -1283,30 +1393,39 @@ class PollingDataSourceDriver(DataSourceDriver):
                 #   late (or dies and comes back up), DSE can automatically
                 #   send the full table.
                 if tablename in self.state:
-                    self.publish(tablename, self.state[tablename])
+                    # Note(thread-safety): blocking call
+                    self.publish(
+                        tablename, self.state[tablename], use_snapshot=False)
                 else:
-                    self.publish(tablename, set())
+                    # Note(thread-safety): blocking call
+                    self.publish(tablename, set(), use_snapshot=False)
         except Exception as e:
             self.last_error = e
             LOG.exception("Datasource driver raised exception")
 
         self.last_updated_time = datetime.datetime.now()
         self.number_of_updates += 1
-        self.log_info("finished polling")
+        LOG.info("%s:: finished polling", self.name)
 
+    # Note(thread-safety): blocking function
     def request_refresh(self):
         """Request a refresh of this service's data."""
         try:
+            # Note(thread-safety): blocking call
             self.refresh_request_queue.put(None)
         except eventlet.queue.Full:
             # if the queue is full, just ignore it, the poller thread will
             # get to it eventually
             pass
 
+    # Note(thread-safety): blocking function
     def block_unless_refresh_requested(self):
+        # Note(thread-safety): blocking call
         self.refresh_request_queue.get()
+        # Note(thread-safety): blocking call
         self.poll()
 
+    # Note(thread-safety): blocking function
     def poll_loop(self, poll_time):
         """Entrypoint for the datasource driver's poller greenthread.
 
@@ -1316,17 +1435,22 @@ class PollingDataSourceDriver(DataSourceDriver):
         :param poll_time: is the amount of time (in seconds) to wait between
         polling rounds.
         """
-        while self.running:
+        LOG.debug("start to poll from datasource %s", self.name)
+        while self._running:
             if poll_time:
                 if self.last_updated_time is None:
+                    # Note(thread-safety): blocking call
                     self.poll()
                 else:
                     try:
                         with eventlet.Timeout(poll_time):
+                            # Note(thread-safety): blocking call
                             self.block_unless_refresh_requested()
                     except eventlet.Timeout:
+                        # Note(thread-safety): blocking call
                         self.poll()
             else:
+                # Note(thread-safety): blocking call
                 self.block_unless_refresh_requested()
 
 
@@ -1344,6 +1468,22 @@ class ExecutionDriver(object):
     def __init__(self):
         # a list of action methods which can be used with "execute"
         self.executable_methods = {}
+        self.LEADER_TIMEOUT = 5
+        self._leader_node_id = None
+        # defined in DataService class
+        self.heartbeat_callbacks['check_leader'] = self._check_leader_heartbeat
+
+    def _check_leader_heartbeat(self):
+        """Vacate leader if heartbeat lost"""
+        if (self._leader_node_id is not None and
+           self._leader_node_id != self.node.node_id):
+            peers = self.node.dse_status()['peers']
+            if (self._leader_node_id not in peers or
+               time.time() - peers[self._leader_node_id]['last_hb_time']
+               > self.LEADER_TIMEOUT):
+                LOG.debug('local leader %s vacated due to lost heartbeat',
+                          self._leader_node_id)
+                self._leader_node_id = None
 
     def reqhandler(self, msg):
         """Request handler.
@@ -1360,7 +1500,7 @@ class ExecutionDriver(object):
         try:
             self.execute(action, action_args)
         except Exception as e:
-            LOG.exception(e.message)
+            LOG.exception(str(e))
 
     def add_executable_client_methods(self, client, api_prefix):
         """Inspect client to get supported builtin methods
@@ -1393,14 +1533,16 @@ class ExecutionDriver(object):
         method = reduce(getattr, method_name.split('.'), client)
         return method
 
+    # Note(thread-safety): blocking function (potentially)
     def _execute_api(self, client, action, action_args):
         positional_args = action_args.get('positional', [])
         named_args = action_args.get('named', {})
         LOG.debug('Processing action execution: action = %s, '
-                  'positional args = %s, named args = %s'
-                  % (action, positional_args, named_args))
+                  'positional args = %s, named args = %s',
+                  action, positional_args, named_args)
         try:
             method = self._get_method(client, action)
+            # Note(thread-safety): blocking call (potentially)
             method(*positional_args, **named_args)
         except Exception as e:
             LOG.exception(e)
@@ -1431,6 +1573,24 @@ class ExecutionDriver(object):
                             'description': self.executable_methods[method][1]})
         return {'results': actions}
 
+    # Note(thread-safety): blocking function
+    def request_execute(self, context, action, action_args, wait):
+        """Accept execution requests and execute requests from leader"""
+        node_id = context.get('node_id', None)
+        th = None
+        if self._leader_node_id == node_id:
+            # Note(thread-safety): blocking call
+            th = eventlet.spawn(self.execute, action, action_args)
+        elif node_id is not None:
+            if self._leader_node_id is None:
+                self._leader_node_id = node_id
+                LOG.debug('New local leader %s selected', self._leader_node_id)
+                # Note(thread-safety): blocking call
+                th = eventlet.spawn(self.execute, action, action_args)
+        if wait and th:
+            th.wait()
+
+    # Note(thread-safety): blocking function (in some subclasses)
     def execute(self, action, action_args):
         """This method must be implemented by each driver.
 
