@@ -12,9 +12,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
+
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
+
 import inspect
 
-import keystoneclient.v2_0.client as ksclient
 import muranoclient.client
 from muranoclient.common import exceptions as murano_exceptions
 from oslo_log import log as logging
@@ -30,12 +34,7 @@ from congress import utils
 logger = logging.getLogger(__name__)
 
 
-def d6service(name, keys, inbox, datapath, args):
-    """This method is called by d6cage to create a dataservice instance."""
-    return MuranoDriver(name, keys, inbox, datapath, args)
-
-
-class MuranoDriver(datasource_driver.DataSourceDriver,
+class MuranoDriver(datasource_driver.PollingDataSourceDriver,
                    datasource_driver.ExecutionDriver):
     OBJECTS = "objects"
     PARENT_TYPES = "parent_types"
@@ -48,22 +47,18 @@ class MuranoDriver(datasource_driver.DataSourceDriver,
     UNUSED_ENV_PROPERTIES = ['id', 'tenant_id']
     APPS_TYPE_PREFIXES = ['io.murano.apps', 'io.murano.databases']
 
-    def __init__(self, name='', keys='', inbox=None, datapath=None, args=None):
-        super(MuranoDriver, self).__init__(name, keys, inbox, datapath, args)
+    def __init__(self, name='', args=None):
+        super(MuranoDriver, self).__init__(name, args=args)
         datasource_driver.ExecutionDriver.__init__(self)
         self.creds = args
-        logger.debug("Credentials = %s" % self.creds)
-        keystone = ksclient.Client(**self.creds)
-        murano_endpoint = keystone.service_catalog.url_for(
-            service_type='application_catalog',
-            endpoint_type='publicURL')
-        logger.debug("murano_endpoint = %s" % murano_endpoint)
+        session = datasource_utils.get_keystone_session(self.creds)
         client_version = "1"
         self.murano_client = muranoclient.client.Client(
-            client_version,
-            endpoint=murano_endpoint,
-            token=keystone.auth_token)
-        self.inspect_builtin_methods(self.murano_client, 'muranoclient.v1.')
+            client_version, session=session, endpoint_type='publicURL',
+            service_type='application-catalog')
+        self.add_executable_client_methods(
+            self.murano_client,
+            'muranoclient.v1.')
         logger.debug("Successfully created murano_client")
 
         self.action_call_returns = []
@@ -107,13 +102,8 @@ class MuranoDriver(datasource_driver.DataSourceDriver,
             self._translate_services(environments)
             self._translate_deployments(environments)
             self._translate_connected()
-        except murano_exceptions.HTTPException as e:
-            if e.code == 401:
-                logger.debug("Obtain keystone token again")
-                keystone = ksclient.Client(**self.creds)
-                self.murano_client.auth_token = keystone.auth_token
-            else:
-                raise e
+        except murano_exceptions.HTTPException:
+            raise
 
     @classmethod
     def get_schema(cls):
@@ -427,7 +417,7 @@ class MuranoDriver(datasource_driver.DataSourceDriver,
 
         :param class_name: <string> service/application class name
             e.g. io.murano.apps.linux.Telnet.
-        :return - package type (e.g. 'Application') if found.
+        :return: - package type (e.g. 'Application') if found.
             - None if no package type found.
         """
         pkg_type = None
@@ -517,4 +507,4 @@ class MuranoDriver(datasource_driver.DataSourceDriver,
             action_name = positional_args[2]
             self._call_murano_action(env_id, obj_id, action_name)
         except Exception as e:
-            logger.exception(e.message)
+            logger.exception(str(e))
