@@ -19,7 +19,6 @@ from __future__ import absolute_import
 
 
 from oslo_db import exception as oslo_db_exc
-from oslo_log import log as logging
 import sqlalchemy as sa
 from sqlalchemy.orm import exc as db_exc
 
@@ -27,8 +26,6 @@ from sqlalchemy.orm import exc as db_exc
 from congress.db import api as db
 from congress.db import model_base
 
-LOG = logging.getLogger(__name__)
-
 
 class Policy(model_base.BASE, model_base.HasId, model_base.HasAudit):
     __tablename__ = 'policies'
@@ -94,7 +91,20 @@ class PolicyDeleted(model_base.BASE, model_base.HasId, model_base.HasAudit):
 
 def add_policy(id_, name, abbreviation, description, owner, kind,
                deleted=False, session=None):
-    session = session or db.get_session()
+    if session:
+        # IMPORTANT: if session provided, do not interrupt existing transaction
+        # with BEGIN which can drop db locks and change desired transaction
+        # boundaries for proper commit and rollback
+        try:
+            policy = Policy(id_, name, abbreviation, description, owner,
+                            kind, deleted)
+            session.add(policy)
+            return policy
+        except oslo_db_exc.DBDuplicateEntry:
+            raise KeyError("Policy with name %s already exists" % name)
+
+    # else
+    session = db.get_session()
     try:
         with session.begin(subtransactions=True):
             policy = Policy(id_, name, abbreviation, description, owner,
@@ -209,7 +219,17 @@ class PolicyRule(model_base.BASE, model_base.HasId, model_base.HasAudit):
 
 def add_policy_rule(id, policy_name, rule, comment, deleted=False,
                     rule_name="", session=None):
-    session = session or db.get_session()
+    if session:
+        # IMPORTANT: if session provided, do not interrupt existing transaction
+        # with BEGIN which can drop db locks and change desired transaction
+        # boundaries for proper commit and rollback
+        policy_rule = PolicyRule(id, policy_name, rule, comment,
+                                 deleted, rule_name=rule_name)
+        session.add(policy_rule)
+        return policy_rule
+
+    # else
+    session = db.get_session()
     with session.begin(subtransactions=True):
         policy_rule = PolicyRule(id, policy_name, rule, comment,
                                  deleted, rule_name=rule_name)
