@@ -15,12 +15,17 @@
 #    under the License.
 
 """Congress base exception handling."""
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
 
 import sys
 
 from oslo_config import cfg
 from oslo_log import log as logging
+import six
 
+from congress.api import error_codes
 from congress import utils
 
 
@@ -61,8 +66,8 @@ class CongressException(Exception):
         :param data will contain specifics for this instance of the
         exception, e.g. a description error message.
         """
-        self.data = kwargs.pop('data', None)
-        self.name = kwargs.pop('name', None)
+        self.data = kwargs.get('data', None)
+        self.name = kwargs.get('name', None)
 
         # TODO(thinrichs): remove the rest of this (except the call to super)
         self.kwargs = kwargs
@@ -73,23 +78,27 @@ class CongressException(Exception):
                 pass
 
         if not message:
-            try:
-                message = self.msg_fmt % kwargs
+            if self.name is not None:
+                error_code = error_codes.get_num(self.name)
+                description = error_codes.get_desc(self.name)
+                message = "(%s) %s" % (error_code, description)
+            else:
+                try:
+                    message = self.msg_fmt % kwargs
 
-            except Exception:
-                exc_info = sys.exc_info()
-                # kwargs doesn't match a variable in the message
-                # log the issue and the kwargs
-                LOG.exception(_('Exception in string format operation'))
-                for name, value in kwargs.items():
-                    LOG.error("%s: %s", name, value)    # noqa
-
-                if CONF.fatal_exception_format_errors:
-                    raise exc_info[0], exc_info[1], exc_info[2]
-                else:
-                    # at least get the core message out if something happened
-                    message = self.msg_fmt
+                except Exception:
+                    exc_info = sys.exc_info()
+                    # kwargs doesn't match a variable in the message
+                    # log the issue and the kwargs
+                    LOG.exception(_('Exception in string format operation'))
+                    for name, value in kwargs.items():
+                        LOG.error("%s: %s", name, value)    # noqa
 
+                    if CONF.fatal_exception_format_errors:
+                        six.reraise(exc_info[0], exc_info[1], exc_info[2])
+                    else:
+                        # at least get the core message out
+                        message = self.msg_fmt
         super(CongressException, self).__init__(message)
 
     def format_message(self):
@@ -97,6 +106,7 @@ class CongressException(Exception):
         # which should be our full CongressException message, (see __init__)
         return self.args[0]
 
+
 # FIXME(thinrichs): Get rid of the ones below and instead create exception
 #   classes to represent the parts of the code that generated the exception,
 #   e.g. datasources versus policy compiler versus policy runtime.
@@ -146,11 +156,19 @@ class DanglingReference(Conflict):
     pass
 
 
+class LazyTable(BadRequest):
+    msg_fmt = _("table %(lazy_table)s is a lazy table and is not subscribed.")
+
+
+class InvalidPolicyInput(BadRequest):
+    msg_fmt = _('Input policy item violates schema.')
+
+
 # NOTE(thinrichs): The following represent different kinds of
 #   exceptions: the policy compiler and the policy runtime, respectively.
 class PolicyException(CongressException):
     def __init__(self, msg=None, obj=None, line=None, col=None,
-                 name=None, data=None):
+                 name=None, data=None, **kwargs):
         CongressException.__init__(self, message=msg, name=name, data=data)
         self.obj = obj
         self.location = utils.Location(line=line, col=col, obj=obj)
@@ -164,3 +182,61 @@ class PolicyException(CongressException):
 
 class PolicyRuntimeException(CongressException):
     pass
+
+
+class DatabaseError(CongressException):
+    msg_fmt = _("Database backend experienced an unknown error.")
+
+
+class DatabaseDataError(BadRequest):
+    msg_fmt = _("Database backend experienced a data error. "
+                "E.g. value out of range, text string too long.")
+
+
+class IncompleteSchemaException(CongressException):
+    pass
+
+
+class DataServiceError(Exception):
+    pass
+
+
+class BadConfig(BadRequest):
+    pass
+
+
+class DatasourceDriverException(CongressException):
+    pass
+
+
+class MissingRequiredConfigOptions(BadConfig):
+    msg_fmt = _("Missing required config options: %(missing_options)s")
+
+
+class InvalidDriver(BadConfig):
+    msg_fmt = _("Invalid driver: %(driver)s")
+
+
+class InvalidDriverOption(BadConfig):
+    msg_fmt = _("Invalid driver options: %(invalid_options)s")
+
+
+class DatasourceNameInUse(Conflict):
+    msg_fmt = _("Datasource already in use with name %(value)s")
+
+
+class InvalidDatasourceName(BadConfig):
+    msg_fmt = _("Datasource name %(value) is invalid. Cannot be empty or "
+                "start with underscore.  Must be valid in policy language")
+
+
+class DatasourceNotFound(NotFound):
+    msg_fmt = _("Datasource not found %(id)s")
+
+
+class DriverNotFound(NotFound):
+    msg_fmt = _("Driver not found %(id)s")
+
+
+class DatasourceCreationError(BadConfig):
+    msg_fmt = _("Datasource could not be created on the DSE: %(value)s")
