@@ -40,6 +40,7 @@ from congress.db import db_policy_rules
 from congress.dse2 import data_service
 from congress import exception
 from congress.synchronizer import policy_rule_synchronizer
+from congress import utils
 
 LOG = logging.getLogger(__name__)
 
@@ -244,9 +245,26 @@ class Runtime (object):
     ###############################################
     # Persistence layer
     ###############################################
+    # Note(thread-safety): blocking function
+    def persistent_create_policy_with_rules(self, policy_rules_obj):
+        rules, policy_metadata = self.persistent_insert_rules(
+            policy_name=policy_rules_obj['name'],
+            rules=policy_rules_obj['rules'],
+            create_policy=True,
+            abbr=policy_rules_obj.get('abbreviation'),
+            kind=policy_rules_obj.get('kind'),
+            desc=policy_rules_obj.get('description'))
+
+        # remove the rule IDs
+        for rule in rules:
+            del rule['id']
+
+        policy_metadata['rules'] = rules
+        return policy_metadata
+
     # Note(thread-safety): blocking function
     def persistent_create_policy(self, name, id_=None, abbr=None, kind=None,
-                                 desc=None):
+                                 desc=None, db_session=None):
         # validation for name
         if not compile.string_is_servicename(name):
             raise exception.PolicyException(
@@ -274,7 +292,8 @@ class Runtime (object):
                                        obj['abbreviation'],
                                        obj['description'],
                                        obj['owner_id'],
-                                       obj['kind'])
+                                       obj['kind'],
+                                       session=db_session)
         except KeyError:
             raise
         except Exception:
@@ -282,7 +301,13 @@ class Runtime (object):
             msg = "Error thrown while adding policy %s into DB." % policy_name
             LOG.exception(msg)
             raise exception.PolicyException(msg)
-        self.synchronizer.sync_one_policy(obj['name'])
+        if db_session:
+            # stay in current transaction, previous write may not be
+            # readable by synchronizer
+            self.add_policy_obj_to_runtime(policy_obj)
+        else:
+            self.synchronizer.sync_one_policy(obj['name'],
+                                              db_session=db_session)
         return obj
 
     # Note(thread-safety): blocking function
@@ -331,107 +356,180 @@ class Runtime (object):
         rules = db_policy_rules.get_policy_rules(policy_name)
         return [rule.to_dict() for rule in rules]
 
-    # Note(thread-safety): blocking function
-    # acquire lock to avoid periodic sync from undoing insert before persisted
-    @lockutils.synchronized('congress_synchronize_rules')
     def persistent_insert_rule(self, policy_name, str_rule, rule_name,
                                comment):
+        rule_data = {'rule': str_rule, 'name': rule_name,
+                     'comment': comment}
+        return_data, _ = self.persistent_insert_rules(policy_name, [rule_data])
+        return (return_data[0]['id'], return_data[0])
+
+    # Note(thread-safety): blocking function
+    # acquire lock to avoid periodic sync from undoing insert before persisted
+    # IMPORTANT: Be very careful to avoid deadlock when
+    # acquiring locks sequentially. In this case, we will acquire lock A
+    # then attempt to acquire lock B. We have to make sure no thread will hold
+    # lock B and attempt to acquire lock A, causing a deadlock
+    @lockutils.synchronized('congress_synchronize_policies')
+    @lockutils.synchronized('congress_synchronize_rules')
+    def persistent_insert_rules(self, policy_name, rules, create_policy=False,
+                                id_=None, abbr=None, kind=None, desc=None):
         """Insert and persists rule into policy_name."""
+
+        def uninsert_rules(rules_inserted):
+            for d in rules_inserted:
+                self._safe_process_policy_update(
+                    [d['input_rule_str']], policy_name, insert=False)
+
+        success = False  # used to rollback DB if not set to success
         try:
+            rules_to_persist = []
+            return_data = []
+            # get session
+            db_session = db_api.get_locking_session()
+
+            # lock policy_rules table to prevent conflicting rules
+            # insertion (say causing unsupported recursion)
+            # policies and datasources tables locked because
+            # it's a requirement of MySQL backend to lock all accessed tables
+            db_api.lock_tables(session=db_session,
+                               tables=['policy_rules', 'policies',
+                                       'datasources'])
+
             if cfg.CONF.replicated_policy_engine:
-                # get session
-                db_session = db_api.get_locking_session()
-                db_session.begin(subtransactions=True)
-
-                # lock policy_rules table to prevent conflicting rules
-                # insertion (say causing unsupported recursion)
-                db_api.lock_tables(session=db_session, tables=['policy_rules'])
-
                 # synchronize policy rules to get latest state, locked state
                 # non-locking version because lock already acquired,
                 # avoid deadlock
                 self.synchronizer.synchronize_rules_nonlocking(
                     db_session=db_session)
-            else:
-                db_session = None
 
-            # Reject rules inserted into non-persisted policies
-            # (i.e. datasource policies)
-            # Note(thread-safety): blocking call
-            policy_name = db_policy_rules.policy_name(
-                policy_name)
-            # call synchronizer to make sure policy is synchronized in memory
-            self.synchronizer.sync_one_policy(policy_name)
-            # Note(thread-safety): blocking call
-            policies = db_policy_rules.get_policies()
-            persisted_policies = set([p.name for p in policies])
-            if policy_name not in persisted_policies:
-                if policy_name in self.theory:
-                    LOG.debug(
-                        "insert_persisted_rule error: rule not permitted for "
-                        "policy %s", policy_name)
+            # Note: it's important that this create policy is run after locking
+            # the policy_rules table, so as to prevent other nodes from
+            # inserting rules into this policy, which may be removed by an
+            # undo (delete the policy) later in this method
+            policy_metadata = None
+            if create_policy:
+                policy_metadata = self.persistent_create_policy(
+                    id_=id_, name=policy_name, abbr=abbr, kind=kind,
+                    desc=desc, db_session=db_session)
+            else:
+                # Reject rules inserted into non-persisted policies
+                # (i.e. datasource policies)
+
+                # Note(thread-safety): blocking call
+                policy_name = db_policy_rules.policy_name(
+                    policy_name, session=db_session)
+                # call synchronizer to make sure policy is sync'ed in memory
+                self.synchronizer.sync_one_policy_nonlocking(
+                    policy_name, db_session=db_session)
+                # Note(thread-safety): blocking call
+                policies = db_policy_rules.get_policies(session=db_session)
+                persisted_policies = set([p.name for p in policies])
+                if policy_name not in persisted_policies:
+                    if policy_name in self.theory:
+                        LOG.debug(
+                            "insert_persisted_rule error: rule not permitted "
+                            "for policy %s", policy_name)
+                        raise exception.PolicyRuntimeException(
+                            name='rule_not_permitted')
+
+            rules_to_insert = []
+            for rule_data in rules:
+                str_rule = rule_data['rule']
+                rule_name = rule_data.get('name')
+                comment = rule_data.get('comment')
+
+                id_ = uuidutils.generate_uuid()
+                try:
+                    rule = self.parse(str_rule)
+                except exception.PolicyException as e:
+                    # TODO(thinrichs): change compiler to provide these
+                    # error_code names directly.
+                    raise exception.PolicyException(
+                        str(e), name='rule_syntax')
+
+                if len(rule) == 1:
+                    rule = rule[0]
+                else:
+                    msg = ("Received multiple rules: " +
+                           "; ".join(str(x) for x in rule))
                     raise exception.PolicyRuntimeException(
-                        name='rule_not_permitted')
+                        msg, name='multiple_rules')
 
-            id_ = uuidutils.generate_uuid()
-            try:
-                rule = self.parse(str_rule)
-            except exception.PolicyException as e:
-                # TODO(thinrichs): change compiler to provide these error_code
-                #   names directly.
-                raise exception.PolicyException(str(e), name='rule_syntax')
-
-            if len(rule) == 1:
-                rule = rule[0]
-            else:
-                msg = ("Received multiple rules: " +
-                       "; ".join(str(x) for x in rule))
-                raise exception.PolicyRuntimeException(
-                    msg, name='multiple_rules')
-
-            rule.set_id(id_)
-            rule.set_name(rule_name)
-            rule.set_comment(comment or "")
-            rule.set_original_str(str_rule)
+                rule.set_id(id_)
+                rule.set_name(rule_name)
+                rule.set_comment(comment or "")
+                rule.set_original_str(str_rule)
+                rules_to_insert.append(rule)
             changes = self._safe_process_policy_update(
-                rule, policy_name, persistent=True)
+                rules_to_insert, policy_name, persistent=True)
+
+            if len(changes) > 0:
+                # remember the rule for possible undo
+                rules_inserted = [
+                    change_event.formula for change_event in changes]
+
+                # remember the rule for insert into DB
+                rules_to_persist = [{
+                    'original_str': change_event.formula.original_str,
+                    'id': str(change_event.formula.id),
+                    'comment': change_event.formula.comment,
+                    'name': change_event.formula.name}
+                    for change_event in changes]
+
+                # prepare return data based on rules inserted
+                return_data = [{
+                    'rule': utils.pretty_rule(
+                        change_event.formula.original_str),
+                    'id': str(change_event.formula.id),
+                    'comment': change_event.formula.comment,
+                    'name': change_event.formula.name}
+                    for change_event in changes]
+
             # save rule to database if change actually happened.
             # Note: change produced may not be equivalent to original rule
             #   because of column-reference elimination.
-            if len(changes) > 0:
-                d = {'rule': rule.pretty_str(),
-                     'id': str(rule.id),
-                     'comment': rule.comment,
-                     'name': rule.name}
-                try:
+            if len(rules_to_persist) == 0 and len(rules) > 0:
+                # change not accepted means it was already there
+                raise exception.PolicyRuntimeException(
+                    name='rule_already_exists')
+            try:
+                for d in rules_to_persist:
                     # Note(thread-safety): blocking call
                     db_policy_rules.add_policy_rule(
-                        d['id'], policy_name, str_rule, d['comment'],
-                        rule_name=d['name'], session=db_session)
+                        d['id'], policy_name, d['original_str'],
+                        d['comment'], rule_name=d['name'], session=db_session)
                     # do not begin to avoid implicitly releasing table
                     # lock due to starting new transaction
-                    return (d['id'], d)
-                except Exception as db_exception:
-                    try:
-                        self._safe_process_policy_update(
-                            rule, policy_name, insert=False)
-                        raise exception.PolicyRuntimeException(
-                            "Error while writing to DB: %s."
-                            % str(db_exception))
-                    except Exception as change_exception:
-                        raise exception.PolicyRuntimeException(
-                            "Error thrown during recovery from DB error. "
-                            "Inconsistent state.  DB error: %s.  "
-                            "New error: %s." % (str(db_exception),
-                                                str(change_exception)))
-
-            # change not accepted means it was already there
-            raise exception.PolicyRuntimeException(
-                name='rule_already_exists')
+                success = True
+                return return_data, policy_metadata
+            except Exception as db_exception:
+                try:
+                    # un-insert all rules from engine unless all db inserts
+                    # succeeded
+                    # Note limitation: if an unexpected DB error is encountered
+                    # the rule insertions into policy engine are undone, but
+                    # may have already had effects on actions and query results
+                    uninsert_rules(rules_inserted)
+                    raise exception.PolicyRuntimeException(
+                        "Error while writing to DB: %s."
+                        % str(db_exception))
+                except Exception as change_exception:
+                    raise exception.PolicyRuntimeException(
+                        "Error thrown during recovery from DB error. "
+                        "Inconsistent state.  DB error: %s.  "
+                        "New error: %s." % (str(db_exception),
+                                            str(change_exception)))
         finally:
-            # commit, unlock, and close db_session
+            # commit/rollback, unlock, and close db_session
             if db_session:
-                db_api.commit_unlock_tables(session=db_session)
+                if success:
+                    db_api.commit_unlock_tables(session=db_session)
+                else:
+                    db_api.rollback_unlock_tables(session=db_session)
+                    if create_policy:
+                        # sync the potentially rolled back policy creation
+                        self.synchronizer.sync_one_policy_nonlocking(
+                            policy_name)
                 db_session.close()
 
     # Note(thread-safety): blocking function
@@ -445,7 +543,7 @@ class Runtime (object):
                 name='rule_not_exists',
                 data='ID: %s, policy_name: %s' % (id_, policy_name))
         rule = self.parse1(item['rule'])
-        self._safe_process_policy_update(rule, policy_name, insert=False)
+        self._safe_process_policy_update([rule], policy_name, insert=False)
         # Note(thread-safety): blocking call
         db_policy_rules.delete_policy_rule(id_)
         return item
@@ -466,21 +564,20 @@ class Runtime (object):
             parsed_rule.set_comment(rule.comment)
             parsed_rule.set_original_str(rule.rule)
             self._safe_process_policy_update(
-                parsed_rule,
+                [parsed_rule],
                 rule.policy_name)
 
-    def _safe_process_policy_update(self, parsed_rule, policy_name,
+    def _safe_process_policy_update(self, parsed_rules, policy_name,
                                     insert=True, persistent=False):
         if policy_name not in self.theory:
             raise exception.PolicyRuntimeException(
                 'Policy ID %s does not exist' % policy_name,
                 name='policy_not_exist')
-        event = compile.Event(
-            formula=parsed_rule,
-            insert=insert,
-            target=policy_name)
+        events = [compile.Event(
+            formula=parsed_rule, insert=insert, target=policy_name)
+            for parsed_rule in parsed_rules]
         (permitted, changes) = self.process_policy_update(
-            [event], persistent=persistent)
+            events, persistent=persistent)
         if not permitted:
             raise exception.PolicyException(
                 ";".join([str(x) for x in changes]),
@@ -1056,18 +1153,20 @@ class Runtime (object):
         enabled = []
         errors = []
         for event in events:
-            errs = compile.check_schema_consistency(
-                event.formula, self.theory, event.target)
-            if len(errs) > 0:
-                errors.append((event, errs))
-                continue
             try:
                 oldformula = event.formula
-                event.formula = oldformula.eliminate_column_references(
-                    self.theory, default_theory=event.target)
+                event.formula = \
+                    oldformula.eliminate_column_references_and_pad_positional(
+                        self.theory, default_theory=event.target)
                 # doesn't copy over ID since it creates a new one
                 event.formula.set_id(oldformula.id)
                 enabled.append(event)
+
+                errs = compile.check_schema_consistency(
+                    event.formula, self.theory, event.target)
+                if len(errs) > 0:
+                    errors.append((event, errs))
+                    continue
             except exception.IncompleteSchemaException as e:
                 if persistent:
                     # FIXME(ekcs): inconsistent behavior?
@@ -2019,7 +2118,9 @@ class DseRuntime (Runtime, data_service.DataService):
         """Overloading the DseRuntime version of _rpc so it uses dse2."""
         # TODO(ramineni): This is called only during execute_action, added
         # the same function name for compatibility with old arch
-        args = {'action': action, 'action_args': args}
+
+        retry_rpc = cfg.CONF.dse.execute_action_retry
+        args = {'action': action, 'action_args': args, 'wait': retry_rpc}
 
         def execute_once():
             return self.rpc(service_name, 'request_execute', args,
@@ -2043,7 +2144,7 @@ class DseRuntime (Runtime, data_service.DataService):
                       action, args['action_args'])
 
         # long timeout for action execution because actions can take a while
-        if not cfg.CONF.dse.execute_action_retry:
+        if not retry_rpc:
             # Note(thread-safety): blocking call
             #   Only when thread pool at capacity
             eventlet.spawn_n(execute_once)
@@ -2172,6 +2273,11 @@ class DseRuntimeEndpoints(object):
         # Note(thread-safety): blocking call
         return self.dse.persistent_create_policy(name, id_, abbr, kind, desc)
 
+    # Note(thread-safety): blocking function
+    def persistent_create_policy_with_rules(self, context, policy_rules_obj):
+        # Note(thread-safety): blocking call
+        return self.dse.persistent_create_policy_with_rules(policy_rules_obj)
+
     # Note(thread-safety): blocking function
     def persistent_delete_policy(self, context, name_or_id):
         # Note(thread-safety): blocking call
