@@ -17,8 +17,8 @@ from oslo_log import log as logging
 from tempest import clients
 from tempest import config
 from tempest.lib.common.utils import test_utils
+from tempest.lib import decorators
 from tempest.lib import exceptions
-from tempest import test
 
 from congress_tempest_tests.tests.scenario import manager_congress
 
@@ -34,7 +34,7 @@ class TestCinderDriver(manager_congress.ScenarioPolicyBase):
         super(TestCinderDriver, cls).skip_checks()
         if not (CONF.network.project_networks_reachable or
                 CONF.network.public_network_id):
-            msg = ('Either project_networks_reachable must be "true", or'
+            msg = ('Either project_networks_reachable must be "true", or '
                    'public_network_id must be defined.')
             cls.enabled = False
             raise cls.skipException(msg)
@@ -45,18 +45,19 @@ class TestCinderDriver(manager_congress.ScenarioPolicyBase):
 
     def setUp(cls):
         super(TestCinderDriver, cls).setUp()
-        cls.os = clients.Manager(cls.admin_manager.auth_provider.credentials)
-        cls.cinder = cls.os.volumes_v2_client
+        cls.os_primary = clients.Manager(
+            cls.os_admin.auth_provider.credentials)
+        cls.cinder = cls.os_primary.volumes_v2_client
         cls.datasource_id = manager_congress.get_datasource_id(
-            cls.admin_manager.congress_client, 'cinder')
+            cls.os_admin.congress_client, 'cinder')
         res = cls.cinder.create_volume(size=1, description=None, name='v0',
                                        consistencygroup_id=None, metadata={})
         LOG.debug('result of creating new volume: %s', res)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_cinder_volumes_table(self):
         volume_schema = (
-            self.admin_manager.congress_client.show_datasource_table_schema(
+            self.os_admin.congress_client.show_datasource_table_schema(
                 self.datasource_id, 'volumes')['columns'])
         volume_id_col = next(i for i, c in enumerate(volume_schema)
                              if c['name'] == 'id')
@@ -71,7 +72,7 @@ class TestCinderDriver(manager_congress.ScenarioPolicyBase):
                 volumes_map[volume['id']] = volume
 
             results = (
-                self.admin_manager.congress_client.list_datasource_rows(
+                self.os_admin.congress_client.list_datasource_rows(
                     self.datasource_id, 'volumes'))
             LOG.debug('congress cinder volumes table: %s', results)
             # check that congress and cinder return the same volume IDs
@@ -106,7 +107,7 @@ class TestCinderDriver(manager_congress.ScenarioPolicyBase):
             raise exceptions.TimeoutException("Data did not converge in time "
                                               "or failure in server")
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_update_no_error(self):
         if not test_utils.call_until_true(
                 func=lambda: self.check_datasource_no_error('cinder'),
