@@ -13,16 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
 from tempest import clients
 from tempest import config
+from tempest.lib.common.utils import test_utils
+from tempest.lib import decorators
 from tempest.lib import exceptions
 from tempest import test
 
 from congress_tempest_tests.tests.scenario import manager_congress
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
 
 
 class TestGlanceV2Driver(manager_congress.ScenarioPolicyBase):
@@ -43,16 +43,17 @@ class TestGlanceV2Driver(manager_congress.ScenarioPolicyBase):
 
     def setUp(cls):
         super(TestGlanceV2Driver, cls).setUp()
-        cls.os = clients.Manager(cls.admin_manager.auth_provider.credentials)
-        cls.glancev2 = cls.os.image_client_v2
+        cls.os_primary = clients.Manager(
+            cls.os_admin.auth_provider.credentials)
+        cls.glancev2 = cls.os_primary.image_client_v2
         cls.datasource_id = manager_congress.get_datasource_id(
-            cls.admin_manager.congress_client, 'glancev2')
+            cls.os_admin.congress_client, 'glancev2')
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     @test.services('image')
     def test_glancev2_images_table(self):
         image_schema = (
-            self.admin_manager.congress_client.show_datasource_table_schema(
+            self.os_admin.congress_client.show_datasource_table_schema(
                 self.datasource_id, 'images')['columns'])
         image_id_col = next(i for i, c in enumerate(image_schema)
                             if c['name'] == 'id')
@@ -66,7 +67,7 @@ class TestGlanceV2Driver(manager_congress.ScenarioPolicyBase):
                 image_map[image['id']] = image
 
             results = (
-                self.admin_manager.congress_client.list_datasource_rows(
+                self.os_admin.congress_client.list_datasource_rows(
                     self.datasource_id, 'images'))
             for row in results['results']:
                 try:
@@ -89,12 +90,13 @@ class TestGlanceV2Driver(manager_congress.ScenarioPolicyBase):
                         return False
             return True
 
-        if not test.call_until_true(func=_check_data_table_glancev2_images,
-                                    duration=100, sleep_for=4):
+        if not test_utils.call_until_true(
+                func=_check_data_table_glancev2_images,
+                duration=100, sleep_for=4):
             raise exceptions.TimeoutException("Data did not converge in time "
                                               "or failure in server")
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     @test.services('image')
     def test_glancev2_tags_table(self):
         def _check_data_table_glance_images():
@@ -106,7 +108,7 @@ class TestGlanceV2Driver(manager_congress.ScenarioPolicyBase):
                 image_tag_map[image['id']] = image['tags']
 
             results = (
-                self.admin_manager.congress_client.list_datasource_rows(
+                self.os_admin.congress_client.list_datasource_rows(
                     self.datasource_id, 'tags'))
             for row in results['results']:
                 image_id, tag = row['data'][0], row['data'][1]
@@ -119,7 +121,16 @@ class TestGlanceV2Driver(manager_congress.ScenarioPolicyBase):
                     return False
             return True
 
-        if not test.call_until_true(func=_check_data_table_glance_images,
-                                    duration=100, sleep_for=5):
+        if not test_utils.call_until_true(
+                func=_check_data_table_glance_images,
+                duration=100, sleep_for=5):
             raise exceptions.TimeoutException("Data did not converge in time "
                                               "or failure in server")
+
+    @decorators.attr(type='smoke')
+    def test_update_no_error(self):
+        if not test_utils.call_until_true(
+                func=lambda: self.check_datasource_no_error('glancev2'),
+                duration=30, sleep_for=5):
+            raise exceptions.TimeoutException('Datasource could not poll '
+                                              'without error.')
