@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
 from tempest import config
 from tempest.lib.common.utils import test_utils
+from tempest.lib import decorators
 from tempest.lib import exceptions
 from tempest import test
 
@@ -23,7 +23,6 @@ from congress_tempest_tests.tests.scenario import helper
 from congress_tempest_tests.tests.scenario import manager_congress
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
 
 
 class TestNovaDriver(manager_congress.ScenarioPolicyBase):
@@ -47,15 +46,15 @@ class TestNovaDriver(manager_congress.ScenarioPolicyBase):
         self.keypairs = {}
         self.servers = []
         self.datasource_id = manager_congress.get_datasource_id(
-            self.admin_manager.congress_client, 'nova')
+            self.os_admin.congress_client, 'nova')
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     @test.services('compute', 'network')
     def test_nova_datasource_driver_servers(self):
         self._setup_network_and_servers()
 
         server_schema = (
-            self.admin_manager.congress_client.show_datasource_table_schema(
+            self.os_admin.congress_client.show_datasource_table_schema(
                 self.datasource_id, 'servers')['columns'])
         # Convert some of the column names.
 
@@ -78,7 +77,7 @@ class TestNovaDriver(manager_congress.ScenarioPolicyBase):
         @helper.retry_on_exception
         def _check_data_table_nova_servers():
             results = (
-                self.admin_manager.congress_client.list_datasource_rows(
+                self.os_admin.congress_client.list_datasource_rows(
                     self.datasource_id, 'servers'))
             for row in results['results']:
                 match = True
@@ -105,7 +104,7 @@ class TestNovaDriver(manager_congress.ScenarioPolicyBase):
             raise exceptions.TimeoutException("Data did not converge in time "
                                               "or failure in server")
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     @test.services('compute', 'network')
     def test_nova_datasource_driver_flavors(self):
 
@@ -119,7 +118,7 @@ class TestNovaDriver(manager_congress.ScenarioPolicyBase):
                 flavor_id_map[flavor['id']] = flavor
 
             results = (
-                self.admin_manager.congress_client.list_datasource_rows(
+                self.os_admin.congress_client.list_datasource_rows(
                     self.datasource_id, 'flavors'))
             # TODO(alexsyip): Not sure what the following OS-FLV-EXT-DATA:
             # prefix is for.
@@ -144,7 +143,7 @@ class TestNovaDriver(manager_congress.ScenarioPolicyBase):
             raise exceptions.TimeoutException("Data did not converge in time "
                                               "or failure in server")
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_update_no_error(self):
         if not test_utils.call_until_true(
                 func=lambda: self.check_datasource_no_error('nova'),
