@@ -51,6 +51,10 @@ function configure_congress {
 
     cp $CONGRESS_DIR/etc/api-paste.ini $CONGRESS_API_PASTE_FILE
     cp $CONGRESS_DIR/etc/policy.json $CONGRESS_POLICY_FILE
+    if [[ ! -d $CONGRESS_LIBRARY_DIR ]]; then
+        mkdir $CONGRESS_LIBRARY_DIR
+    fi
+    cp -r $CONGRESS_DIR/library/* $CONGRESS_LIBRARY_DIR
 
     # Update either configuration file
     iniset $CONGRESS_CONF DEFAULT debug $ENABLE_DEBUG_LOG_LEVEL
@@ -66,13 +70,12 @@ function configure_congress {
     CONGRESS_DRIVERS="congress.datasources.neutronv2_driver.NeutronV2Driver,"
     CONGRESS_DRIVERS+="congress.datasources.glancev2_driver.GlanceV2Driver,"
     CONGRESS_DRIVERS+="congress.datasources.nova_driver.NovaDriver,"
-    CONGRESS_DRIVERS+="congress.datasources.keystone_driver.KeystoneDriver,"
+    CONGRESS_DRIVERS+="congress.datasources.keystonev3_driver.KeystoneV3Driver,"
     CONGRESS_DRIVERS+="congress.datasources.ceilometer_driver.CeilometerDriver,"
     CONGRESS_DRIVERS+="congress.datasources.cinder_driver.CinderDriver,"
     CONGRESS_DRIVERS+="congress.datasources.swift_driver.SwiftDriver,"
     CONGRESS_DRIVERS+="congress.datasources.plexxi_driver.PlexxiDriver,"
     CONGRESS_DRIVERS+="congress.datasources.vCenter_driver.VCenterDriver,"
-    CONGRESS_DRIVERS+="congress.datasources.cloudfoundryv2_driver.CloudFoundryV2Driver,"
     CONGRESS_DRIVERS+="congress.datasources.murano_driver.MuranoDriver,"
     CONGRESS_DRIVERS+="congress.datasources.ironic_driver.IronicDriver,"
     CONGRESS_DRIVERS+="congress.datasources.heatv1_driver.HeatV1Driver,"
@@ -86,20 +89,12 @@ function configure_congress {
 
     _congress_setup_keystone $CONGRESS_CONF keystone_authtoken
 
-    if is_service_enabled horizon; then
-        _congress_setup_horizon
-    fi
-    _congress_setup_third_party_requirements
-}
-
-function _congress_setup_third_party_requirements {
-    pip_install -r $CONGRESS_DIR/thirdparty-requirements.txt
 }
 
 function configure_congress_datasources {
     _configure_service neutron neutronv2
     _configure_service nova nova
-    _configure_service key keystone
+    _configure_service key keystonev3
     _configure_service ceilometer ceilometer
     _configure_service cinder cinder
     _configure_service swift swift
@@ -118,7 +113,7 @@ function _configure_service {
             --config username=$OS_USERNAME \
             --config tenant_name=$OS_PROJECT_NAME \
             --config password=$OS_PASSWORD \
-            --config auth_url=http://$SERVICE_HOST:5000/v3
+            --config auth_url=http://$SERVICE_HOST/identity
     fi
 }
 
@@ -132,10 +127,12 @@ function create_predefined_policy {
     fi
 }
 
-
-function configure_congressclient {
-    setup_develop $CONGRESSCLIENT_DIR
+function _install_congress_dashboard {
+    git_clone $CONGRESSDASHBOARD_REPO $CONGRESSDASHBOARD_DIR $CONGRESSDASHBOARD_BRANCH
+    setup_develop $CONGRESSDASHBOARD_DIR
+    _congress_setup_horizon
 }
+
 # create_congress_cache_dir() - Part of the _congress_setup_keystone() process
 function create_congress_cache_dir {
     # Create cache dir
@@ -176,9 +173,19 @@ function init_congress {
 # install_congress() - install dependency, collect client source and prepare
 function install_congress {
     # congress requires java so we install it here
-    install_package default-jre
+    if is_ubuntu; then
+        install_package default-jre
+    elif is_fedora; then
+        install_package jre
+    else
+        die $LINENO "Congress devstack only supports Debian and Red Hat-based"
+    fi
     git_clone $CONGRESSCLIENT_REPO $CONGRESSCLIENT_DIR $CONGRESSCLIENT_BRANCH
     setup_develop $CONGRESSCLIENT_DIR
+
+    if is_service_enabled horizon; then
+        _install_congress_dashboard
+    fi
 }
 
 # Start running processes, including screen
@@ -192,17 +199,18 @@ function start_congress_service_and_check {
 
     if [ "$CONGRESS_MULTIPROCESS_DEPLOYMENT" == "False" ]; then
         echo "Installing congress as single process"
-        run_process congress "python $CONGRESS_BIN_DIR/congress-server --node-id=allinonenode $CFG_FILE_OPTIONS"
+        run_process congress "$CONGRESS_BIN_DIR/congress-server --node-id=allinonenode $CFG_FILE_OPTIONS"
     else
-        run_process congress-api "python $CONGRESS_BIN_DIR/congress-server --api --node-id=apinode $CFG_FILE_OPTIONS"
-        run_process congress-engine "python $CONGRESS_BIN_DIR/congress-server --policy-engine --node-id=enginenode $CFG_FILE_OPTIONS"
-        run_process congress-datasources "python $CONGRESS_BIN_DIR/congress-server --datasources --node-id=datanode $CFG_FILE_OPTIONS"
+        echo "Installing congress as multi process"
+        run_process congress-api "$CONGRESS_BIN_DIR/congress-server --api --node-id=apinode $CFG_FILE_OPTIONS"
+        run_process congress-engine "$CONGRESS_BIN_DIR/congress-server --policy-engine --node-id=enginenode $CFG_FILE_OPTIONS"
+        run_process congress-datasources "$CONGRESS_BIN_DIR/congress-server --datasources --node-id=datanode $CFG_FILE_OPTIONS"
     fi
 
     # Start multiple PE's
     if [ "$CONGRESS_REPLICATED" == "True" ]; then
-        run_process congress-engine "python $CONGRESS_BIN_DIR/congress-server --policy-engine --node-id=enginenode-2 $CFG_FILE_OPTIONS"
-        run_process congress-engine "python $CONGRESS_BIN_DIR/congress-server --policy-engine --node-id=enginenode-3 $CFG_FILE_OPTIONS"
+        run_process congress-engine "$CONGRESS_BIN_DIR/congress-server --policy-engine --node-id=enginenode-2 $CFG_FILE_OPTIONS"
+        run_process congress-engine "$CONGRESS_BIN_DIR/congress-server --policy-engine --node-id=enginenode-3 $CFG_FILE_OPTIONS"
     fi
 
     echo "Waiting for Congress to start..."
@@ -213,6 +221,21 @@ function start_congress_service_and_check {
 #    if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q -O- http://$CONGRESS_HOST:$CONGRESS_PORT; do sleep 1; done"; then
 #        die $LINENO "Congress did not start"
 #    fi
+
+    # Expose encryption key to tempest test launched replica instances
+    # WARNING: this setting deploys an insecure setup meant for gate-test only
+    # Explanation: congress_tempest_tests/tests/scenario/congress_ha/test_ha.py
+    #    launches replica congress instances from a different user than the
+    #    original devstack-launched instance. Hence, the encryption keys need
+    #    to be exposed to additional users for the test to work as intended.
+    # Note: This works correctly only for default encryption key location
+    # /etc/congress/keys
+    # If needed in future, this script can read custom key location from
+    # $CONGRESS_CONF and adjust accordingly
+    if [ "$CONGRESS_EXPOSE_ENCRYPTION_KEY_FOR_TEST" == "True" ]; then
+        chmod a+rx /etc/congress/keys
+        chmod a+r /etc/congress/keys/aes_key
+    fi
 }
 
 
@@ -246,32 +269,10 @@ function _congress_setup_keystone {
 # Set up Horizon integration with Congress
 function _congress_setup_horizon {
     # Dashboard panels
-    cp $CONGRESS_HORIZON_DIR/enabled/_50_policy.py $HORIZON_DIR/openstack_dashboard/local/enabled/
-    cp $CONGRESS_HORIZON_DIR/enabled/_60_policies.py $HORIZON_DIR/openstack_dashboard/local/enabled/
-    cp $CONGRESS_HORIZON_DIR/enabled/_70_datasources.py $HORIZON_DIR/openstack_dashboard/local/enabled/
-
-    # For unit tests
-    echo "python-congressclient" >> $HORIZON_DIR/requirements.txt
-    echo -e \
-"\n# Load the pluggable dashboard settings"\
-"\nimport openstack_dashboard.local.enabled"\
-"\nfrom openstack_dashboard.utils import settings"\
-"\n\nINSTALLED_APPS = list(INSTALLED_APPS)"\
-"\nsettings.update_dashboards(["\
-"\n    openstack_dashboard.local.enabled,"\
-"\n], HORIZON_CONFIG, INSTALLED_APPS)" >> $HORIZON_DIR/openstack_dashboard/test/settings.py
-
-    # Setup alias for django-admin which could be different depending on distro
-    local django_admin
-    if type -p django-admin > /dev/null; then
-        django_admin=django-admin
-    else
-        django_admin=django-admin.py
-    fi
-
-    # Collect and compress static files (e.g., JavaScript, CSS)
-    DJANGO_SETTINGS_MODULE=openstack_dashboard.settings $django_admin collectstatic --noinput
-    DJANGO_SETTINGS_MODULE=openstack_dashboard.settings $django_admin compress --force
+    ln -fs $CONGRESSDASHBOARD_DIR/congress_dashboard/enabled/_50_policy.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -fs $CONGRESSDASHBOARD_DIR/congress_dashboard/enabled/_60_policies.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -fs $CONGRESSDASHBOARD_DIR/congress_dashboard/enabled/_70_datasources.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -fs $CONGRESSDASHBOARD_DIR/congress_dashboard/enabled/_75_monitoring.py $HORIZON_DIR/openstack_dashboard/local/enabled/
 
     # Restart Horizon
     restart_apache_server
@@ -286,7 +287,6 @@ if is_service_enabled congress; then
         install_congress
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring Congress"
-        configure_congressclient
         configure_congress
 
         if is_service_enabled key; then
